/*
 * SPDX-FileCopyrightText: 2021 David Redondo <kde@david-redondo.de>
 *
 * SPDX-License-Identifier: CC0-1.0
 */

#pragma once

constexpr auto helperLocation = "/usr/lib/ksystemstats_intel_helper";
