/****************************************************************************
** Meta object code from reading C++ file 'ksystemstats1adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "ksystemstats1adaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ksystemstats1adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20Ksystemstats1AdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto Ksystemstats1Adaptor::qt_create_metaobjectdata<qt_meta_tag_ZN20Ksystemstats1AdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Ksystemstats1Adaptor",
        "D-Bus Interface",
        "org.kde.ksystemstats1",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.ksystemstats1\">\n    <signal name=\"s"
        "ensorAdded\">\n      <arg direction=\"out\" type=\"s\" name=\"sens"
        "orId\"/>\n    </signal>\n    <signal name=\"sensorRemoved\">\n    "
        "  <arg direction=\"out\" type=\"s\" name=\"sensorId\"/>\n    </sig"
        "nal>\n    <signal name=\"newSensorData\">\n      <arg direction=\""
        "out\" type=\"a(sv)\" name=\"sensorData\"/>\n      <annotation valu"
        "e=\"KSysGuard::SensorDataList\" name=\"org.qtproject.QtDBus.QtType"
        "Name.Out0\"/>\n    </signal>\n    <signal name=\"sensorMetaDataCha"
        "nged\">\n      <arg direction=\"out\" type=\"a{s(sssuuddi)}\" name"
        "=\"metaData\"/>\n      <annotation value=\"QHash&lt;QString,KSysGu"
        "ard::SensorInfo&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\""
        "/>\n    </signal>\n    <method name=\"allSensors\">\n      <arg di"
        "rection=\"out\" type=\"a{s(sssuuddi)}\" name=\"sensorInfo\"/>\n   "
        "   <annotation value=\"QHash&lt;QString,KSysGuard::SensorInfo&gt;\""
        " name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </method>\n "
        "   <method name=\"sensors\">\n      <arg direction=\"in\" type=\"a"
        "s\" name=\"sensorsIds\"/>\n      <arg direction=\"out\" type=\"a{s"
        "(sssuuddi)}\" name=\"sensorInfo\"/>\n      <annotation value=\"QHa"
        "sh&lt;QString,KSysGuard::SensorInfo&gt;\" name=\"org.qtproject.QtD"
        "Bus.QtTypeName.Out0\"/>\n    </method>\n    <method name=\"subscri"
        "be\">\n      <arg direction=\"in\" type=\"as\" name=\"sensorIds\"/"
        ">\n    </method>\n    <method name=\"unsubscribe\">\n      <arg di"
        "rection=\"in\" type=\"as\" name=\"sensorIds\"/>\n    </method>\n  "
        "  <method name=\"sensorData\">\n      <arg direction=\"in\" type=\""
        "as\" name=\"sensorIds\"/>\n      <arg direction=\"out\" type=\"a(s"
        "v)\" name=\"sensorData\"/>\n      <annotation value=\"KSysGuard::S"
        "ensorDataList\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n "
        "   </method>\n  </interface>\n",
        "newSensorData",
        "",
        "KSysGuard::SensorDataList",
        "sensorData",
        "sensorAdded",
        "sensorId",
        "sensorMetaDataChanged",
        "QHash<QString,KSysGuard::SensorInfo>",
        "metaData",
        "sensorRemoved",
        "allSensors",
        "sensorIds",
        "sensors",
        "sensorsIds",
        "subscribe",
        "unsubscribe"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newSensorData'
        QtMocHelpers::SignalData<void(KSysGuard::SensorDataList)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'sensorAdded'
        QtMocHelpers::SignalData<void(const QString &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'sensorMetaDataChanged'
        QtMocHelpers::SignalData<void(const QHash<QString,KSysGuard::SensorInfo> &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'sensorRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'allSensors'
        QtMocHelpers::SlotData<QHash<QString,KSysGuard::SensorInfo>()>(15, 6, QMC::AccessPublic, 0x80000000 | 12),
        // Slot 'sensorData'
        QtMocHelpers::SlotData<KSysGuard::SensorDataList(const QStringList &)>(8, 6, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QStringList, 16 },
        }}),
        // Slot 'sensors'
        QtMocHelpers::SlotData<QHash<QString,KSysGuard::SensorInfo>(const QStringList &)>(17, 6, QMC::AccessPublic, 0x80000000 | 12, {{
            { QMetaType::QStringList, 18 },
        }}),
        // Slot 'subscribe'
        QtMocHelpers::SlotData<void(const QStringList &)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 16 },
        }}),
        // Slot 'unsubscribe'
        QtMocHelpers::SlotData<void(const QStringList &)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Ksystemstats1Adaptor, qt_meta_tag_ZN20Ksystemstats1AdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Ksystemstats1Adaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20Ksystemstats1AdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20Ksystemstats1AdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20Ksystemstats1AdaptorE_t>.metaTypes,
    nullptr
} };

void Ksystemstats1Adaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Ksystemstats1Adaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newSensorData((*reinterpret_cast<std::add_pointer_t<KSysGuard::SensorDataList>>(_a[1]))); break;
        case 1: _t->sensorAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->sensorMetaDataChanged((*reinterpret_cast<std::add_pointer_t<QHash<QString,KSysGuard::SensorInfo>>>(_a[1]))); break;
        case 3: _t->sensorRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: { QHash<QString,KSysGuard::SensorInfo> _r = _t->allSensors();
            if (_a[0]) *reinterpret_cast<QHash<QString,KSysGuard::SensorInfo>*>(_a[0]) = std::move(_r); }  break;
        case 5: { KSysGuard::SensorDataList _r = _t->sensorData((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KSysGuard::SensorDataList*>(_a[0]) = std::move(_r); }  break;
        case 6: { QHash<QString,KSysGuard::SensorInfo> _r = _t->sensors((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QHash<QString,KSysGuard::SensorInfo>*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->subscribe((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 8: _t->unsubscribe((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Ksystemstats1Adaptor::*)(KSysGuard::SensorDataList )>(_a, &Ksystemstats1Adaptor::newSensorData, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Ksystemstats1Adaptor::*)(const QString & )>(_a, &Ksystemstats1Adaptor::sensorAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Ksystemstats1Adaptor::*)(const QHash<QString,KSysGuard::SensorInfo> & )>(_a, &Ksystemstats1Adaptor::sensorMetaDataChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Ksystemstats1Adaptor::*)(const QString & )>(_a, &Ksystemstats1Adaptor::sensorRemoved, 3))
            return;
    }
}

const QMetaObject *Ksystemstats1Adaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Ksystemstats1Adaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20Ksystemstats1AdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int Ksystemstats1Adaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Ksystemstats1Adaptor::newSensorData(KSysGuard::SensorDataList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Ksystemstats1Adaptor::sensorAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Ksystemstats1Adaptor::sensorMetaDataChanged(const QHash<QString,KSysGuard::SensorInfo> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Ksystemstats1Adaptor::sensorRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
