/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.ksystemstats1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "ksystemstats1adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class Ksystemstats1Adaptor
 */

Ksystemstats1Adaptor::Ksystemstats1Adaptor(Daemon *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

Ksystemstats1Adaptor::~Ksystemstats1Adaptor()
{
    // destructor
}

QHash<QString,KSysGuard::SensorInfo> Ksystemstats1Adaptor::allSensors()
{
    // handle method call org.kde.ksystemstats1.allSensors
    return parent()->allSensors();
}

KSysGuard::SensorDataList Ksystemstats1Adaptor::sensorData(const QStringList &sensorIds)
{
    // handle method call org.kde.ksystemstats1.sensorData
    return parent()->sensorData(sensorIds);
}

QHash<QString,KSysGuard::SensorInfo> Ksystemstats1Adaptor::sensors(const QStringList &sensorsIds)
{
    // handle method call org.kde.ksystemstats1.sensors
    return parent()->sensors(sensorsIds);
}

void Ksystemstats1Adaptor::subscribe(const QStringList &sensorIds)
{
    // handle method call org.kde.ksystemstats1.subscribe
    parent()->subscribe(sensorIds);
}

void Ksystemstats1Adaptor::unsubscribe(const QStringList &sensorIds)
{
    // handle method call org.kde.ksystemstats1.unsubscribe
    parent()->unsubscribe(sensorIds);
}


#include "moc_ksystemstats1adaptor.cpp"
