/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.ksystemstats1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef KSYSTEMSTATS1ADAPTOR_H
#define KSYSTEMSTATS1ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "daemon.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.ksystemstats1
 */
class Ksystemstats1Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.ksystemstats1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.ksystemstats1\">\n"
"    <signal name=\"sensorAdded\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"sensorId\"/>\n"
"    </signal>\n"
"    <signal name=\"sensorRemoved\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"sensorId\"/>\n"
"    </signal>\n"
"    <signal name=\"newSensorData\">\n"
"      <arg direction=\"out\" type=\"a(sv)\" name=\"sensorData\"/>\n"
"      <annotation value=\"KSysGuard::SensorDataList\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </signal>\n"
"    <signal name=\"sensorMetaDataChanged\">\n"
"      <arg direction=\"out\" type=\"a{s(sssuuddi)}\" name=\"metaData\"/>\n"
"      <annotation value=\"QHash&lt;QString,KSysGuard::SensorInfo&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </signal>\n"
"    <method name=\"allSensors\">\n"
"      <arg direction=\"out\" type=\"a{s(sssuuddi)}\" name=\"sensorInfo\"/>\n"
"      <annotation value=\"QHash&lt;QString,KSysGuard::SensorInfo&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"sensors\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"sensorsIds\"/>\n"
"      <arg direction=\"out\" type=\"a{s(sssuuddi)}\" name=\"sensorInfo\"/>\n"
"      <annotation value=\"QHash&lt;QString,KSysGuard::SensorInfo&gt;\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"subscribe\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"sensorIds\"/>\n"
"    </method>\n"
"    <method name=\"unsubscribe\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"sensorIds\"/>\n"
"    </method>\n"
"    <method name=\"sensorData\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"sensorIds\"/>\n"
"      <arg direction=\"out\" type=\"a(sv)\" name=\"sensorData\"/>\n"
"      <annotation value=\"KSysGuard::SensorDataList\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    Ksystemstats1Adaptor(Daemon *parent);
    ~Ksystemstats1Adaptor() override;

    inline Daemon *parent() const
    { return static_cast<Daemon *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QHash<QString,KSysGuard::SensorInfo> allSensors();
    KSysGuard::SensorDataList sensorData(const QStringList &sensorIds);
    QHash<QString,KSysGuard::SensorInfo> sensors(const QStringList &sensorsIds);
    void subscribe(const QStringList &sensorIds);
    void unsubscribe(const QStringList &sensorIds);
Q_SIGNALS: // SIGNALS
    void newSensorData(KSysGuard::SensorDataList sensorData);
    void sensorAdded(const QString &sensorId);
    void sensorMetaDataChanged(const QHash<QString,KSysGuard::SensorInfo> &metaData);
    void sensorRemoved(const QString &sensorId);
};

#endif
