/****************************************************************************
** Meta object code from reading C++ file 'daemon.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ksystemstats-6.6.4/src/daemon.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'daemon.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6DaemonE_t {};
} // unnamed namespace

template <> constexpr inline auto Daemon::qt_create_metaobjectdata<qt_meta_tag_ZN6DaemonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Daemon",
        "sensorAdded",
        "",
        "sensorId",
        "sensorRemoved",
        "newSensorData",
        "KSysGuard::SensorDataList",
        "sensorData",
        "allSensors",
        "KSysGuard::SensorInfoMap",
        "sensors",
        "sensorsIds",
        "subscribe",
        "sensorIds",
        "unsubscribe"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sensorAdded'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'sensorRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'newSensorData'
        QtMocHelpers::SignalData<void(const KSysGuard::SensorDataList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'allSensors'
        QtMocHelpers::SlotData<KSysGuard::SensorInfoMap() const>(8, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'sensors'
        QtMocHelpers::SlotData<KSysGuard::SensorInfoMap(const QStringList &) const>(10, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QStringList, 11 },
        }}),
        // Slot 'subscribe'
        QtMocHelpers::SlotData<void(const QStringList &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 13 },
        }}),
        // Slot 'unsubscribe'
        QtMocHelpers::SlotData<void(const QStringList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 13 },
        }}),
        // Slot 'sensorData'
        QtMocHelpers::SlotData<KSysGuard::SensorDataList(const QStringList &)>(7, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QStringList, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Daemon, qt_meta_tag_ZN6DaemonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Daemon::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DaemonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DaemonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6DaemonE_t>.metaTypes,
    nullptr
} };

void Daemon::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Daemon *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sensorAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->sensorRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->newSensorData((*reinterpret_cast<std::add_pointer_t<KSysGuard::SensorDataList>>(_a[1]))); break;
        case 3: { KSysGuard::SensorInfoMap _r = _t->allSensors();
            if (_a[0]) *reinterpret_cast<KSysGuard::SensorInfoMap*>(_a[0]) = std::move(_r); }  break;
        case 4: { KSysGuard::SensorInfoMap _r = _t->sensors((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KSysGuard::SensorInfoMap*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->subscribe((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 6: _t->unsubscribe((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 7: { KSysGuard::SensorDataList _r = _t->sensorData((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KSysGuard::SensorDataList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KSysGuard::SensorDataList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const QString & )>(_a, &Daemon::sensorAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const QString & )>(_a, &Daemon::sensorRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const KSysGuard::SensorDataList & )>(_a, &Daemon::newSensorData, 2))
            return;
    }
}

const QMetaObject *Daemon::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Daemon::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DaemonE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int Daemon::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Daemon::sensorAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Daemon::sensorRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Daemon::newSensorData(const KSysGuard::SensorDataList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
