// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "pressure_logging.h"


Q_LOGGING_CATEGORY(KSYSTEMSTATS_PRESSURE, "org.kde.ksystemstats.plugins.pressure", QtWarningMsg)

